package Classifier;
use strict;
use warnings;

sub new {
    my $class = shift;
    my $self = bless {}, $class;
    $self->{term_count}      = {};
    $self->{category_count}  = {};
    return $self;
}

sub train {
    my ($self, $vec, $cat) = @_;
    while (my ($term, $count) = each %$vec) {
        $self->{term_count}->{$term}->{$cat} += $count;
        $self->{category_count}->{$cat}++;
    }
}

sub dump {
    my $self = shift;
    warn Data::Dumper::Dumper($self->{term_count}, $self->{category_count});
}

sub cat_prob {
    my ($self, $cat) = @_;
    my $total = 0;
    for my $v (values %{$self->{category_count}}) {
        $total += $v;
    }
    return $self->{category_count}->{$cat} / $total;
}

sub term_count {
    my ($self, $term, $cat) = @_;
    return $self->{term_count}->{$term}->{$cat} || 0;
}

sub term_prob {
    my ($self, $term, $cat) = @_;
    return $self->term_count($term, $cat) / $self->{category_count}->{$cat};
}

sub total_term_count {
    my $self = shift;
    my $count = 0;
    for my $v (values %{$self->{category_count}}) {
        $count += $v;
    }
    return $count;
}

sub score {
    my ($self, $vec, $cat) = @_;
    my $cat_prob = log( $self->cat_prob($cat) );
    my $not_likely = 1 / ($self->total_term_count * 10);
    my $doc_prob = 0;
    while (my ($term, $count) = each %$vec) {
        $doc_prob += log($self->term_prob($term, $cat) || $not_likely) * $count;
    }
    return $cat_prob + $doc_prob;
}

sub predict {
    my ($self, $vec) = @_;
    my %scores;
    for my $cat (keys %{$self->{category_count}}) {
        $scores{$cat} = $self->score($vec, $cat);
    }
    my @classes = sort { $scores{$b} <=> $scores{$a} } keys %scores;
    return $classes[0];
}

1;
